<?php

namespace EventPortal\Tests\Unit\Venue;

use EventPortal\Domain\Address\Models\Address;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\EventVenue\Actions\AssociateToEvent;
use EventPortal\Domain\EventVenue\Actions\DisassociateFromEvent;
use EventPortal\Domain\Venue\Models\Venue;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class VenueTest extends TestCase
{
    use RefreshDatabase;

    /** @test */
    public function can_get_address_from_venue(): void
    {
        $venue = factory(Venue::class)->create(['name' => 'Test Venue']);
        $address = factory(Address::class)->make();

        $venue->address()->save($address);

        $this->assertSameModel($address, $venue->address);
    }

    /** @test */
    public function can_assign_to_event(): void
    {
        $venue = factory(Venue::class)->state('has-address')->create();
        $event = factory(Event::class)->create();

        $this->assertEmpty($venue->events);
        $this->assertEmpty($event->venues);

        $this->associate()->execute($event, [$venue->id]);

        $this->assertContainsModel($venue, $event->fresh()->venues);
        $this->assertContainsModel($event, $venue->fresh()->events);
    }

    /** @test */
    public function can_remove_from_event(): void
    {
        $venue = factory(Venue::class)->state('has-address')->create();
        $event = factory(Event::class)->create();
        $this->associate()->execute($event, [$venue->id]);

        $this->assertContainsModel($venue, $event->fresh()->venues);
        $this->assertContainsModel($event, $venue->fresh()->events);

        $this->disassociate()->execute($event, [$venue->id]);

        $this->assertEmpty($event->fresh()->venues);
    }

    private function associate(): AssociateToEvent
    {
        return app(AssociateToEvent::class);
    }

    private function disassociate(): DisassociateFromEvent
    {
        return app(DisassociateFromEvent::class);
    }
}
